/*---------------------------------------------------------
File Name       : usart.c
Function        : Init, send and receive data with the USART
Created         : 10.12.2006
Version         : 0.02
Supported MCU   : ATmega8, ATmega16, ATmega32
                : ATtiny2313
-----------------------------------------------------------*/

#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "usart.h"

//----------------------------------------------------------
// Function    : usart_init
// Description : Init the USART with Baudrate, Charsize, Parity and Stopbit
// Syntax      : usart_init(19200, 8, 'N', 1);
//----------------------------------------------------------
void usart_init(uint32_t baud, uint8_t charsize, char parity, uint8_t stopbits)
{
  baud = F_CPU/(16*baud)-1;
	UBRR0L = (uint8_t)baud;
	#ifdef UBRR0H
	UBRR0H = (uint8_t)(baud>>8);
	#endif

    // Init temp var for UCSRC
    #ifdef URSEL0
    uint8_t i=(1<<URSEL0);
    #else
    uint8_t i=0;
    #endif

    if (stopbits==2)
    {
        i=(1<<USBS0);
    }

    switch(charsize)
    {
        case 6 : i|=(1<<UCSZ00);
            break;
        case 7 : i|=(1<<UCSZ01);
            break;
        case 8 : i|=(1<<UCSZ00)|(1<<UCSZ01);
            break;
        case 9 : i|=(1<<UCSZ00)|(1<<UCSZ01)|(1<<UCSZ02);
            break;
    }

    switch(parity)
    {
        case 'E': i|=(1<<UPM01);
            break;
        case 'O': i|=(1<<UPM00);
            break;
    }

    UCSR0C=i;

    UCSR0B=(1<<RXEN0)|(1<<TXEN0);
}

//----------------------------------------------------------
// Function     : usart_putc
// Description  : Transmit a byte or char. No buffer, no IRQ
// Syntax       : usart_putc(0xff); usart_putc('a');
// Return       : none 
//----------------------------------------------------------
void usart_putc( uint8_t data )
{
    while ( !( UCSR0A & (1<<UDRE0)) );
    UDR0 = data;
}

//----------------------------------------------------------
// Function     : usart_getc
// Description  : Receive a byte or char. No buffer, no IRQ
// Syntax       : var=usart_getc();
// Return       : Received char 
//----------------------------------------------------------
uint8_t usart_getc( void )
{
    while ( !(UCSR0A & (1<<RXC0)) );
    return UDR0;
}

//----------------------------------------------------------
// Function     : usart_write
// Description  : Transmit a string
// Syntax       : usart_write("String");
// Return       : none
//----------------------------------------------------------
void usart_write(const char *str )
{
    while (*str)
      usart_putc(*str++);
}

//----------------------------------------------------------
// Function     : usart_writeln
// Description  : Transmit a string with LF or CR/LF
// Syntax       : usart_writeln("String",TRUE); to send CR/LF
// Syntax       : usart_writeln("String",FALSE); to send LF
// Return       : none
//----------------------------------------------------------
void usart_writeln(const char *str, char CR )
{
    while (*str)
      usart_putc(*str++);
    if(CR)usart_putc('\r');
    usart_putc('\n');
}

//----------------------------------------------------------
// Function     : usart_writepgm
// Description  : Transmit a string
// Syntax       : usart_writepgm("String");
// Return       : none
//----------------------------------------------------------
void usart_writepgm(const char *str )
{
    register uint8_t chr;
    while ( (chr = pgm_read_byte(str++)) )
      usart_putc(chr);
}

//----------------------------------------------------------
// Function     : usart_writepgm
// Description  : Transmit a string
// Syntax       : usart_writepgm("String");
// Return       : none
//----------------------------------------------------------
void usart_writelnpgm(const char *str, char CR )
{
    uint8_t chr;
    while ( (chr = pgm_read_byte(str++)) )
      usart_putc(chr);
    if(CR)usart_putc('\r');
    usart_putc('\n');
}

//----------------------------------------------------------
// Function     : usart_rxstat
// Description  : Returns True if new Data is availible
// Syntax       : var=usart_rxstat();
// Return       : True for Data availible
//              : False for empty Buffer
//----------------------------------------------------------
char usart_rxstat(void)
{
    if((UCSR0A & (1<<RXC0)))
        return TRUE;
    else
        return FALSE;
}

